/*
 * The contents of this file are subject to the MonetDB Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.monetdb.org/Legal/MonetDBLicense
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is the MonetDB Database System.
 *
 * The Initial Developer of the Original Code is CWI.
 * Portions created by CWI are Copyright (C) 1997-July 2008 CWI.
 * Copyright August 2008-2011 MonetDB B.V.
 * All Rights Reserved.
 */

#ifndef _GDK_SCANSELECT_H
#define _GDK_SCANSELECT_H

/*
 * @- "Tools"
 * First, we need some tools. These are the type-expanded replacements
 * for some generic macros from gdk.mx, gdk_bat.mx, and gdk_atom.mx.
 * Eventually, they should be moved there to be used in other places as well.
 *
 * VOIDput & SIMPLEput should go to gdk_atoms where ATOMput is defined.
 */
#ifdef NOEXPAND_CHR
#ifndef NOEXPAND_BIT
#define NOEXPAND_BIT 1
#endif
#ifndef NOEXPAND_BTE
#define NOEXPAND_BTE 1
#endif
#endif

#ifdef NOEXPAND_INT
#ifndef NOEXPAND_FLT
#define NOEXPAND_FLT 1
#endif
#ifndef NOEXPAND_BAT
#define NOEXPAND_BAT 1
#endif
#if SIZEOF_OID == SIZEOF_INT && !defined(NOEXPAND_OID)
#define NOEXPAND_OID 1
#endif
#if SIZEOF_WRD == SIZEOF_INT && !defined(NOEXPAND_WRD)
#define NOEXPAND_WRD 1
#endif
#if SIZEOF_PTR == SIZEOF_INT && !defined(NOEXPAND_PTR)
#define NOEXPAND_PTR 1
#endif
#endif

#ifdef NOEXPAND_LNG
#ifndef NOEXPAND_DBL
#define NOEXPAND_DBL 1
#endif
#if SIZEOF_OID == SIZEOF_LNG && !defined(NOEXPAND_OID)
#define NOEXPAND_OID 1
#endif
#if SIZEOF_WRD == SIZEOF_LNG && !defined(NOEXPAND_WRD)
#define NOEXPAND_WRD 1
#endif
#if SIZEOF_PTR == SIZEOF_LNG && !defined(NOEXPAND_PTR)
#define NOEXPAND_PTR 1
#endif
#endif

#if defined(NOEXPAND_CHR) || defined(SKIP_TYPE_EXPANSIONS)
#define ATOM_PUT_CHR(tpe,hp,dst,p,src)		\
	if (tpe == TYPE_chr) {			\
		((chr*) dst)[p] = *(chr*) src;	\
	} else
#else
#define ATOM_PUT_CHR(tpe,hp,dst,p,src)
#endif
#if defined(NOEXPAND_BTE) || defined(SKIP_TYPE_EXPANSIONS)
#define ATOM_PUT_BTE(tpe,hp,dst,p,src)		\
	if (tpe == TYPE_bte) {			\
		*(bte*) dst = *(bte*) src;	\
	} else
#else
#define ATOM_PUT_BTE(tpe,hp,dst,p,src)
#endif
#if defined(NOEXPAND_SHT) || defined(SKIP_TYPE_EXPANSIONS)
#define ATOM_PUT_SHT(tpe,hp,dst,p,src)		\
	if (tpe == TYPE_sht) {			\
		((sht*) dst)[p] = *(sht*) src;	\
	} else
#else
#define ATOM_PUT_SHT(tpe,hp,dst,p,src)
#endif
#if defined(NOEXPAND_INT) || defined(SKIP_TYPE_EXPANSIONS)
#define ATOM_PUT_INT(tpe,hp,dst,p,src)		\
	if (tpe == TYPE_int) {			\
		((int*) dst)[p] = *(int*) src;	\
	} else
#else
#define ATOM_PUT_INT(tpe,hp,dst,p,src)
#endif
#if defined(NOEXPAND_LNG) || defined(SKIP_TYPE_EXPANSIONS)
#define ATOM_PUT_LNG(tpe,hp,dst,p,src)		\
	if (tpe == TYPE_lng) {			\
		((lng*) dst)[p] = *(lng*) src;	\
	} else
#else
#define ATOM_PUT_LNG(tpe,hp,dst,p,src)
#endif

#define FIXEDATOM_PUT(b,tpe,p,src,SRC)					\
	do {								\
		ATOM_PUT_CHR(tpe,(b)->SRC->vheap,(b)->SRC->heap.base,(p),(src)) \
		ATOM_PUT_SHT(tpe,(b)->SRC->vheap,(b)->SRC->heap.base,(p),(src)) \
		ATOM_PUT_INT(tpe,(b)->SRC->vheap,(b)->SRC->heap.base,(p),(src)) \
		ATOM_PUT_LNG(tpe,(b)->SRC->vheap,(b)->SRC->heap.base,(p),(src)) \
		if (tpe == TYPE_bat) {					\
			BBPincref(* (bat *) (src), TRUE);			\
			((bat*) (b)->SRC->heap.base)[p] = *(bat*) (src);			\
		} else {						\
			memcpy(SRC##loc((b),(p)), (src), (size_t) ATOMsize(tpe)); \
		}							\
	} while (0)
#define VARATOM_PUT(b,tpe,p,src,SRC) SRC##putvalue((b),SRC##loc((b),(p)),(src),1)
#define SIMPLE_PUT(b,tpe,p,src,SRC) ((tpe*)((b)->SRC->heap.base))[p] = *(tpe*)(src)
#define VOID_PUT(b,tpe,p,src,SRC)
#define VID_PUT(b,tpe,p,src,SRC) ((oid*)((b)->SRC->heap.base))[p] = (src)



/*
 * @-
 * Expand on head type
 */
/*
 * @-
 * Type-specific
 * HT_bunfastins_nocheck_noinc, HT_bunfastins_nocheck & HT_bunfastins
 * should go to gdk.mx, and
 * HT_BUNfastins
 * should go to gdk_bat.mx
 * replacing their generic versions there.
 */

	
#define voidvoid_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VOID_PUT(b,void,p,h,H); \
		VOID_PUT(b,void,p,t,T); \
	} while (0)
#define voidvoid_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		voidvoid_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define voidvoid_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		voidvoid_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define voidchr_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VOID_PUT(b,void,p,h,H); \
		SIMPLE_PUT(b,chr,p,t,T); \
	} while (0)
#define voidchr_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		voidchr_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define voidchr_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		voidchr_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define voidbte_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VOID_PUT(b,void,p,h,H); \
		SIMPLE_PUT(b,bte,p,t,T); \
	} while (0)
#define voidbte_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		voidbte_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define voidbte_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		voidbte_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define voidsht_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VOID_PUT(b,void,p,h,H); \
		SIMPLE_PUT(b,sht,p,t,T); \
	} while (0)
#define voidsht_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		voidsht_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define voidsht_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		voidsht_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define voidint_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VOID_PUT(b,void,p,h,H); \
		SIMPLE_PUT(b,int,p,t,T); \
	} while (0)
#define voidint_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		voidint_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define voidint_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		voidint_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define voidflt_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VOID_PUT(b,void,p,h,H); \
		SIMPLE_PUT(b,flt,p,t,T); \
	} while (0)
#define voidflt_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		voidflt_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define voidflt_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		voidflt_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define voidoid_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VOID_PUT(b,void,p,h,H); \
		SIMPLE_PUT(b,oid,p,t,T); \
	} while (0)
#define voidoid_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		voidoid_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define voidoid_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		voidoid_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define voidwrd_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VOID_PUT(b,void,p,h,H); \
		SIMPLE_PUT(b,wrd,p,t,T); \
	} while (0)
#define voidwrd_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		voidwrd_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define voidwrd_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		voidwrd_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define voidlng_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VOID_PUT(b,void,p,h,H); \
		SIMPLE_PUT(b,lng,p,t,T); \
	} while (0)
#define voidlng_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		voidlng_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define voidlng_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		voidlng_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define voiddbl_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VOID_PUT(b,void,p,h,H); \
		SIMPLE_PUT(b,dbl,p,t,T); \
	} while (0)
#define voiddbl_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		voiddbl_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define voiddbl_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		voiddbl_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define voidfix_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VOID_PUT(b,void,p,h,H); \
		FIXEDATOM_PUT(b,(b)->ttype,p,t,T); \
	} while (0)
#define voidfix_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		voidfix_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define voidfix_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		voidfix_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define voidvar_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VOID_PUT(b,void,p,h,H); \
		VARATOM_PUT(b,(b)->ttype,p,t,T); \
	} while (0)
#define voidvar_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		voidvar_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define voidvar_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		voidvar_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)








	
#define chrvoid_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,chr,p,h,H); \
		VOID_PUT(b,void,p,t,T); \
	} while (0)
#define chrvoid_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		chrvoid_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define chrvoid_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		chrvoid_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define chrchr_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,chr,p,h,H); \
		SIMPLE_PUT(b,chr,p,t,T); \
	} while (0)
#define chrchr_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		chrchr_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define chrchr_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		chrchr_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define chrbte_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,chr,p,h,H); \
		SIMPLE_PUT(b,bte,p,t,T); \
	} while (0)
#define chrbte_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		chrbte_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define chrbte_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		chrbte_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define chrsht_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,chr,p,h,H); \
		SIMPLE_PUT(b,sht,p,t,T); \
	} while (0)
#define chrsht_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		chrsht_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define chrsht_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		chrsht_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define chrint_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,chr,p,h,H); \
		SIMPLE_PUT(b,int,p,t,T); \
	} while (0)
#define chrint_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		chrint_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define chrint_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		chrint_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define chrflt_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,chr,p,h,H); \
		SIMPLE_PUT(b,flt,p,t,T); \
	} while (0)
#define chrflt_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		chrflt_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define chrflt_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		chrflt_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define chroid_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,chr,p,h,H); \
		SIMPLE_PUT(b,oid,p,t,T); \
	} while (0)
#define chroid_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		chroid_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define chroid_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		chroid_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define chrwrd_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,chr,p,h,H); \
		SIMPLE_PUT(b,wrd,p,t,T); \
	} while (0)
#define chrwrd_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		chrwrd_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define chrwrd_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		chrwrd_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define chrlng_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,chr,p,h,H); \
		SIMPLE_PUT(b,lng,p,t,T); \
	} while (0)
#define chrlng_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		chrlng_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define chrlng_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		chrlng_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define chrdbl_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,chr,p,h,H); \
		SIMPLE_PUT(b,dbl,p,t,T); \
	} while (0)
#define chrdbl_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		chrdbl_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define chrdbl_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		chrdbl_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define chrfix_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,chr,p,h,H); \
		FIXEDATOM_PUT(b,(b)->ttype,p,t,T); \
	} while (0)
#define chrfix_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		chrfix_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define chrfix_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		chrfix_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define chrvar_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,chr,p,h,H); \
		VARATOM_PUT(b,(b)->ttype,p,t,T); \
	} while (0)
#define chrvar_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		chrvar_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define chrvar_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		chrvar_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)








	
#define btevoid_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,bte,p,h,H); \
		VOID_PUT(b,void,p,t,T); \
	} while (0)
#define btevoid_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		btevoid_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define btevoid_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		btevoid_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define btechr_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,bte,p,h,H); \
		SIMPLE_PUT(b,chr,p,t,T); \
	} while (0)
#define btechr_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		btechr_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define btechr_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		btechr_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define btebte_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,bte,p,h,H); \
		SIMPLE_PUT(b,bte,p,t,T); \
	} while (0)
#define btebte_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		btebte_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define btebte_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		btebte_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define btesht_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,bte,p,h,H); \
		SIMPLE_PUT(b,sht,p,t,T); \
	} while (0)
#define btesht_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		btesht_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define btesht_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		btesht_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define bteint_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,bte,p,h,H); \
		SIMPLE_PUT(b,int,p,t,T); \
	} while (0)
#define bteint_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		bteint_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define bteint_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		bteint_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define bteflt_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,bte,p,h,H); \
		SIMPLE_PUT(b,flt,p,t,T); \
	} while (0)
#define bteflt_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		bteflt_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define bteflt_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		bteflt_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define bteoid_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,bte,p,h,H); \
		SIMPLE_PUT(b,oid,p,t,T); \
	} while (0)
#define bteoid_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		bteoid_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define bteoid_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		bteoid_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define btewrd_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,bte,p,h,H); \
		SIMPLE_PUT(b,wrd,p,t,T); \
	} while (0)
#define btewrd_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		btewrd_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define btewrd_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		btewrd_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define btelng_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,bte,p,h,H); \
		SIMPLE_PUT(b,lng,p,t,T); \
	} while (0)
#define btelng_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		btelng_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define btelng_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		btelng_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define btedbl_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,bte,p,h,H); \
		SIMPLE_PUT(b,dbl,p,t,T); \
	} while (0)
#define btedbl_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		btedbl_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define btedbl_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		btedbl_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define btefix_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,bte,p,h,H); \
		FIXEDATOM_PUT(b,(b)->ttype,p,t,T); \
	} while (0)
#define btefix_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		btefix_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define btefix_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		btefix_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define btevar_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,bte,p,h,H); \
		VARATOM_PUT(b,(b)->ttype,p,t,T); \
	} while (0)
#define btevar_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		btevar_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define btevar_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		btevar_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)








	
#define shtvoid_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,sht,p,h,H); \
		VOID_PUT(b,void,p,t,T); \
	} while (0)
#define shtvoid_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		shtvoid_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define shtvoid_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		shtvoid_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define shtchr_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,sht,p,h,H); \
		SIMPLE_PUT(b,chr,p,t,T); \
	} while (0)
#define shtchr_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		shtchr_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define shtchr_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		shtchr_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define shtbte_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,sht,p,h,H); \
		SIMPLE_PUT(b,bte,p,t,T); \
	} while (0)
#define shtbte_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		shtbte_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define shtbte_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		shtbte_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define shtsht_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,sht,p,h,H); \
		SIMPLE_PUT(b,sht,p,t,T); \
	} while (0)
#define shtsht_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		shtsht_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define shtsht_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		shtsht_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define shtint_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,sht,p,h,H); \
		SIMPLE_PUT(b,int,p,t,T); \
	} while (0)
#define shtint_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		shtint_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define shtint_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		shtint_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define shtflt_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,sht,p,h,H); \
		SIMPLE_PUT(b,flt,p,t,T); \
	} while (0)
#define shtflt_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		shtflt_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define shtflt_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		shtflt_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define shtoid_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,sht,p,h,H); \
		SIMPLE_PUT(b,oid,p,t,T); \
	} while (0)
#define shtoid_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		shtoid_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define shtoid_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		shtoid_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define shtwrd_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,sht,p,h,H); \
		SIMPLE_PUT(b,wrd,p,t,T); \
	} while (0)
#define shtwrd_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		shtwrd_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define shtwrd_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		shtwrd_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define shtlng_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,sht,p,h,H); \
		SIMPLE_PUT(b,lng,p,t,T); \
	} while (0)
#define shtlng_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		shtlng_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define shtlng_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		shtlng_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define shtdbl_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,sht,p,h,H); \
		SIMPLE_PUT(b,dbl,p,t,T); \
	} while (0)
#define shtdbl_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		shtdbl_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define shtdbl_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		shtdbl_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define shtfix_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,sht,p,h,H); \
		FIXEDATOM_PUT(b,(b)->ttype,p,t,T); \
	} while (0)
#define shtfix_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		shtfix_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define shtfix_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		shtfix_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define shtvar_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,sht,p,h,H); \
		VARATOM_PUT(b,(b)->ttype,p,t,T); \
	} while (0)
#define shtvar_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		shtvar_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define shtvar_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		shtvar_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)








	
#define intvoid_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,int,p,h,H); \
		VOID_PUT(b,void,p,t,T); \
	} while (0)
#define intvoid_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		intvoid_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define intvoid_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		intvoid_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define intchr_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,int,p,h,H); \
		SIMPLE_PUT(b,chr,p,t,T); \
	} while (0)
#define intchr_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		intchr_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define intchr_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		intchr_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define intbte_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,int,p,h,H); \
		SIMPLE_PUT(b,bte,p,t,T); \
	} while (0)
#define intbte_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		intbte_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define intbte_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		intbte_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define intsht_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,int,p,h,H); \
		SIMPLE_PUT(b,sht,p,t,T); \
	} while (0)
#define intsht_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		intsht_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define intsht_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		intsht_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define intint_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,int,p,h,H); \
		SIMPLE_PUT(b,int,p,t,T); \
	} while (0)
#define intint_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		intint_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define intint_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		intint_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define intflt_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,int,p,h,H); \
		SIMPLE_PUT(b,flt,p,t,T); \
	} while (0)
#define intflt_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		intflt_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define intflt_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		intflt_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define intoid_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,int,p,h,H); \
		SIMPLE_PUT(b,oid,p,t,T); \
	} while (0)
#define intoid_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		intoid_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define intoid_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		intoid_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define intwrd_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,int,p,h,H); \
		SIMPLE_PUT(b,wrd,p,t,T); \
	} while (0)
#define intwrd_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		intwrd_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define intwrd_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		intwrd_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define intlng_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,int,p,h,H); \
		SIMPLE_PUT(b,lng,p,t,T); \
	} while (0)
#define intlng_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		intlng_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define intlng_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		intlng_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define intdbl_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,int,p,h,H); \
		SIMPLE_PUT(b,dbl,p,t,T); \
	} while (0)
#define intdbl_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		intdbl_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define intdbl_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		intdbl_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define intfix_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,int,p,h,H); \
		FIXEDATOM_PUT(b,(b)->ttype,p,t,T); \
	} while (0)
#define intfix_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		intfix_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define intfix_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		intfix_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define intvar_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,int,p,h,H); \
		VARATOM_PUT(b,(b)->ttype,p,t,T); \
	} while (0)
#define intvar_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		intvar_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define intvar_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		intvar_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)








	
#define fltvoid_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,flt,p,h,H); \
		VOID_PUT(b,void,p,t,T); \
	} while (0)
#define fltvoid_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		fltvoid_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define fltvoid_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		fltvoid_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define fltchr_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,flt,p,h,H); \
		SIMPLE_PUT(b,chr,p,t,T); \
	} while (0)
#define fltchr_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		fltchr_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define fltchr_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		fltchr_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define fltbte_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,flt,p,h,H); \
		SIMPLE_PUT(b,bte,p,t,T); \
	} while (0)
#define fltbte_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		fltbte_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define fltbte_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		fltbte_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define fltsht_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,flt,p,h,H); \
		SIMPLE_PUT(b,sht,p,t,T); \
	} while (0)
#define fltsht_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		fltsht_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define fltsht_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		fltsht_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define fltint_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,flt,p,h,H); \
		SIMPLE_PUT(b,int,p,t,T); \
	} while (0)
#define fltint_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		fltint_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define fltint_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		fltint_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define fltflt_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,flt,p,h,H); \
		SIMPLE_PUT(b,flt,p,t,T); \
	} while (0)
#define fltflt_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		fltflt_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define fltflt_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		fltflt_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define fltoid_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,flt,p,h,H); \
		SIMPLE_PUT(b,oid,p,t,T); \
	} while (0)
#define fltoid_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		fltoid_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define fltoid_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		fltoid_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define fltwrd_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,flt,p,h,H); \
		SIMPLE_PUT(b,wrd,p,t,T); \
	} while (0)
#define fltwrd_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		fltwrd_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define fltwrd_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		fltwrd_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define fltlng_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,flt,p,h,H); \
		SIMPLE_PUT(b,lng,p,t,T); \
	} while (0)
#define fltlng_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		fltlng_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define fltlng_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		fltlng_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define fltdbl_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,flt,p,h,H); \
		SIMPLE_PUT(b,dbl,p,t,T); \
	} while (0)
#define fltdbl_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		fltdbl_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define fltdbl_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		fltdbl_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define fltfix_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,flt,p,h,H); \
		FIXEDATOM_PUT(b,(b)->ttype,p,t,T); \
	} while (0)
#define fltfix_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		fltfix_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define fltfix_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		fltfix_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define fltvar_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,flt,p,h,H); \
		VARATOM_PUT(b,(b)->ttype,p,t,T); \
	} while (0)
#define fltvar_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		fltvar_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define fltvar_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		fltvar_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)








	
#define oidvoid_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,oid,p,h,H); \
		VOID_PUT(b,void,p,t,T); \
	} while (0)
#define oidvoid_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		oidvoid_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define oidvoid_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		oidvoid_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define oidchr_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,oid,p,h,H); \
		SIMPLE_PUT(b,chr,p,t,T); \
	} while (0)
#define oidchr_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		oidchr_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define oidchr_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		oidchr_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define oidbte_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,oid,p,h,H); \
		SIMPLE_PUT(b,bte,p,t,T); \
	} while (0)
#define oidbte_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		oidbte_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define oidbte_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		oidbte_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define oidsht_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,oid,p,h,H); \
		SIMPLE_PUT(b,sht,p,t,T); \
	} while (0)
#define oidsht_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		oidsht_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define oidsht_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		oidsht_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define oidint_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,oid,p,h,H); \
		SIMPLE_PUT(b,int,p,t,T); \
	} while (0)
#define oidint_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		oidint_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define oidint_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		oidint_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define oidflt_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,oid,p,h,H); \
		SIMPLE_PUT(b,flt,p,t,T); \
	} while (0)
#define oidflt_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		oidflt_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define oidflt_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		oidflt_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define oidoid_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,oid,p,h,H); \
		SIMPLE_PUT(b,oid,p,t,T); \
	} while (0)
#define oidoid_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		oidoid_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define oidoid_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		oidoid_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define oidwrd_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,oid,p,h,H); \
		SIMPLE_PUT(b,wrd,p,t,T); \
	} while (0)
#define oidwrd_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		oidwrd_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define oidwrd_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		oidwrd_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define oidlng_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,oid,p,h,H); \
		SIMPLE_PUT(b,lng,p,t,T); \
	} while (0)
#define oidlng_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		oidlng_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define oidlng_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		oidlng_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define oiddbl_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,oid,p,h,H); \
		SIMPLE_PUT(b,dbl,p,t,T); \
	} while (0)
#define oiddbl_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		oiddbl_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define oiddbl_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		oiddbl_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define oidfix_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,oid,p,h,H); \
		FIXEDATOM_PUT(b,(b)->ttype,p,t,T); \
	} while (0)
#define oidfix_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		oidfix_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define oidfix_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		oidfix_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define oidvar_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,oid,p,h,H); \
		VARATOM_PUT(b,(b)->ttype,p,t,T); \
	} while (0)
#define oidvar_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		oidvar_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define oidvar_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		oidvar_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)








	
#define wrdvoid_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,wrd,p,h,H); \
		VOID_PUT(b,void,p,t,T); \
	} while (0)
#define wrdvoid_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		wrdvoid_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define wrdvoid_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		wrdvoid_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define wrdchr_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,wrd,p,h,H); \
		SIMPLE_PUT(b,chr,p,t,T); \
	} while (0)
#define wrdchr_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		wrdchr_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define wrdchr_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		wrdchr_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define wrdbte_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,wrd,p,h,H); \
		SIMPLE_PUT(b,bte,p,t,T); \
	} while (0)
#define wrdbte_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		wrdbte_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define wrdbte_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		wrdbte_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define wrdsht_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,wrd,p,h,H); \
		SIMPLE_PUT(b,sht,p,t,T); \
	} while (0)
#define wrdsht_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		wrdsht_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define wrdsht_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		wrdsht_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define wrdint_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,wrd,p,h,H); \
		SIMPLE_PUT(b,int,p,t,T); \
	} while (0)
#define wrdint_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		wrdint_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define wrdint_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		wrdint_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define wrdflt_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,wrd,p,h,H); \
		SIMPLE_PUT(b,flt,p,t,T); \
	} while (0)
#define wrdflt_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		wrdflt_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define wrdflt_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		wrdflt_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define wrdoid_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,wrd,p,h,H); \
		SIMPLE_PUT(b,oid,p,t,T); \
	} while (0)
#define wrdoid_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		wrdoid_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define wrdoid_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		wrdoid_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define wrdwrd_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,wrd,p,h,H); \
		SIMPLE_PUT(b,wrd,p,t,T); \
	} while (0)
#define wrdwrd_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		wrdwrd_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define wrdwrd_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		wrdwrd_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define wrdlng_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,wrd,p,h,H); \
		SIMPLE_PUT(b,lng,p,t,T); \
	} while (0)
#define wrdlng_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		wrdlng_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define wrdlng_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		wrdlng_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define wrddbl_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,wrd,p,h,H); \
		SIMPLE_PUT(b,dbl,p,t,T); \
	} while (0)
#define wrddbl_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		wrddbl_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define wrddbl_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		wrddbl_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define wrdfix_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,wrd,p,h,H); \
		FIXEDATOM_PUT(b,(b)->ttype,p,t,T); \
	} while (0)
#define wrdfix_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		wrdfix_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define wrdfix_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		wrdfix_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define wrdvar_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,wrd,p,h,H); \
		VARATOM_PUT(b,(b)->ttype,p,t,T); \
	} while (0)
#define wrdvar_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		wrdvar_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define wrdvar_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		wrdvar_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)








	
#define lngvoid_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,lng,p,h,H); \
		VOID_PUT(b,void,p,t,T); \
	} while (0)
#define lngvoid_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		lngvoid_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define lngvoid_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		lngvoid_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define lngchr_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,lng,p,h,H); \
		SIMPLE_PUT(b,chr,p,t,T); \
	} while (0)
#define lngchr_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		lngchr_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define lngchr_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		lngchr_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define lngbte_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,lng,p,h,H); \
		SIMPLE_PUT(b,bte,p,t,T); \
	} while (0)
#define lngbte_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		lngbte_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define lngbte_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		lngbte_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define lngsht_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,lng,p,h,H); \
		SIMPLE_PUT(b,sht,p,t,T); \
	} while (0)
#define lngsht_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		lngsht_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define lngsht_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		lngsht_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define lngint_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,lng,p,h,H); \
		SIMPLE_PUT(b,int,p,t,T); \
	} while (0)
#define lngint_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		lngint_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define lngint_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		lngint_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define lngflt_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,lng,p,h,H); \
		SIMPLE_PUT(b,flt,p,t,T); \
	} while (0)
#define lngflt_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		lngflt_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define lngflt_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		lngflt_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define lngoid_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,lng,p,h,H); \
		SIMPLE_PUT(b,oid,p,t,T); \
	} while (0)
#define lngoid_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		lngoid_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define lngoid_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		lngoid_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define lngwrd_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,lng,p,h,H); \
		SIMPLE_PUT(b,wrd,p,t,T); \
	} while (0)
#define lngwrd_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		lngwrd_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define lngwrd_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		lngwrd_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define lnglng_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,lng,p,h,H); \
		SIMPLE_PUT(b,lng,p,t,T); \
	} while (0)
#define lnglng_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		lnglng_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define lnglng_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		lnglng_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define lngdbl_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,lng,p,h,H); \
		SIMPLE_PUT(b,dbl,p,t,T); \
	} while (0)
#define lngdbl_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		lngdbl_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define lngdbl_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		lngdbl_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define lngfix_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,lng,p,h,H); \
		FIXEDATOM_PUT(b,(b)->ttype,p,t,T); \
	} while (0)
#define lngfix_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		lngfix_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define lngfix_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		lngfix_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define lngvar_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,lng,p,h,H); \
		VARATOM_PUT(b,(b)->ttype,p,t,T); \
	} while (0)
#define lngvar_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		lngvar_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define lngvar_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		lngvar_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)








	
#define dblvoid_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,dbl,p,h,H); \
		VOID_PUT(b,void,p,t,T); \
	} while (0)
#define dblvoid_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		dblvoid_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define dblvoid_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		dblvoid_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define dblchr_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,dbl,p,h,H); \
		SIMPLE_PUT(b,chr,p,t,T); \
	} while (0)
#define dblchr_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		dblchr_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define dblchr_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		dblchr_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define dblbte_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,dbl,p,h,H); \
		SIMPLE_PUT(b,bte,p,t,T); \
	} while (0)
#define dblbte_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		dblbte_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define dblbte_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		dblbte_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define dblsht_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,dbl,p,h,H); \
		SIMPLE_PUT(b,sht,p,t,T); \
	} while (0)
#define dblsht_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		dblsht_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define dblsht_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		dblsht_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define dblint_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,dbl,p,h,H); \
		SIMPLE_PUT(b,int,p,t,T); \
	} while (0)
#define dblint_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		dblint_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define dblint_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		dblint_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define dblflt_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,dbl,p,h,H); \
		SIMPLE_PUT(b,flt,p,t,T); \
	} while (0)
#define dblflt_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		dblflt_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define dblflt_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		dblflt_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define dbloid_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,dbl,p,h,H); \
		SIMPLE_PUT(b,oid,p,t,T); \
	} while (0)
#define dbloid_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		dbloid_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define dbloid_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		dbloid_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define dblwrd_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,dbl,p,h,H); \
		SIMPLE_PUT(b,wrd,p,t,T); \
	} while (0)
#define dblwrd_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		dblwrd_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define dblwrd_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		dblwrd_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define dbllng_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,dbl,p,h,H); \
		SIMPLE_PUT(b,lng,p,t,T); \
	} while (0)
#define dbllng_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		dbllng_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define dbllng_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		dbllng_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define dbldbl_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,dbl,p,h,H); \
		SIMPLE_PUT(b,dbl,p,t,T); \
	} while (0)
#define dbldbl_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		dbldbl_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define dbldbl_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		dbldbl_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define dblfix_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,dbl,p,h,H); \
		FIXEDATOM_PUT(b,(b)->ttype,p,t,T); \
	} while (0)
#define dblfix_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		dblfix_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define dblfix_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		dblfix_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define dblvar_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		SIMPLE_PUT(b,dbl,p,h,H); \
		VARATOM_PUT(b,(b)->ttype,p,t,T); \
	} while (0)
#define dblvar_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		dblvar_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define dblvar_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		dblvar_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)








	
#define vidvoid_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VID_PUT(b,vid,p,h,H); \
		VOID_PUT(b,void,p,t,T); \
	} while (0)
#define vidvoid_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		vidvoid_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define vidvoid_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		vidvoid_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define vidchr_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VID_PUT(b,vid,p,h,H); \
		SIMPLE_PUT(b,chr,p,t,T); \
	} while (0)
#define vidchr_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		vidchr_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define vidchr_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		vidchr_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define vidbte_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VID_PUT(b,vid,p,h,H); \
		SIMPLE_PUT(b,bte,p,t,T); \
	} while (0)
#define vidbte_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		vidbte_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define vidbte_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		vidbte_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define vidsht_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VID_PUT(b,vid,p,h,H); \
		SIMPLE_PUT(b,sht,p,t,T); \
	} while (0)
#define vidsht_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		vidsht_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define vidsht_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		vidsht_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define vidint_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VID_PUT(b,vid,p,h,H); \
		SIMPLE_PUT(b,int,p,t,T); \
	} while (0)
#define vidint_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		vidint_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define vidint_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		vidint_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define vidflt_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VID_PUT(b,vid,p,h,H); \
		SIMPLE_PUT(b,flt,p,t,T); \
	} while (0)
#define vidflt_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		vidflt_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define vidflt_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		vidflt_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define vidoid_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VID_PUT(b,vid,p,h,H); \
		SIMPLE_PUT(b,oid,p,t,T); \
	} while (0)
#define vidoid_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		vidoid_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define vidoid_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		vidoid_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define vidwrd_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VID_PUT(b,vid,p,h,H); \
		SIMPLE_PUT(b,wrd,p,t,T); \
	} while (0)
#define vidwrd_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		vidwrd_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define vidwrd_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		vidwrd_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define vidlng_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VID_PUT(b,vid,p,h,H); \
		SIMPLE_PUT(b,lng,p,t,T); \
	} while (0)
#define vidlng_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		vidlng_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define vidlng_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		vidlng_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define viddbl_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VID_PUT(b,vid,p,h,H); \
		SIMPLE_PUT(b,dbl,p,t,T); \
	} while (0)
#define viddbl_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		viddbl_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define viddbl_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		viddbl_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define vidfix_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VID_PUT(b,vid,p,h,H); \
		FIXEDATOM_PUT(b,(b)->ttype,p,t,T); \
	} while (0)
#define vidfix_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		vidfix_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define vidfix_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		vidfix_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define vidvar_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VID_PUT(b,vid,p,h,H); \
		VARATOM_PUT(b,(b)->ttype,p,t,T); \
	} while (0)
#define vidvar_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		vidvar_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define vidvar_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		vidvar_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)








	
#define fixvoid_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		FIXEDATOM_PUT(b,(b)->htype,p,h,H); \
		VOID_PUT(b,void,p,t,T); \
	} while (0)
#define fixvoid_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		fixvoid_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define fixvoid_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		fixvoid_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define fixchr_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		FIXEDATOM_PUT(b,(b)->htype,p,h,H); \
		SIMPLE_PUT(b,chr,p,t,T); \
	} while (0)
#define fixchr_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		fixchr_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define fixchr_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		fixchr_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define fixbte_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		FIXEDATOM_PUT(b,(b)->htype,p,h,H); \
		SIMPLE_PUT(b,bte,p,t,T); \
	} while (0)
#define fixbte_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		fixbte_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define fixbte_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		fixbte_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define fixsht_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		FIXEDATOM_PUT(b,(b)->htype,p,h,H); \
		SIMPLE_PUT(b,sht,p,t,T); \
	} while (0)
#define fixsht_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		fixsht_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define fixsht_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		fixsht_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define fixint_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		FIXEDATOM_PUT(b,(b)->htype,p,h,H); \
		SIMPLE_PUT(b,int,p,t,T); \
	} while (0)
#define fixint_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		fixint_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define fixint_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		fixint_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define fixflt_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		FIXEDATOM_PUT(b,(b)->htype,p,h,H); \
		SIMPLE_PUT(b,flt,p,t,T); \
	} while (0)
#define fixflt_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		fixflt_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define fixflt_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		fixflt_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define fixoid_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		FIXEDATOM_PUT(b,(b)->htype,p,h,H); \
		SIMPLE_PUT(b,oid,p,t,T); \
	} while (0)
#define fixoid_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		fixoid_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define fixoid_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		fixoid_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define fixwrd_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		FIXEDATOM_PUT(b,(b)->htype,p,h,H); \
		SIMPLE_PUT(b,wrd,p,t,T); \
	} while (0)
#define fixwrd_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		fixwrd_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define fixwrd_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		fixwrd_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define fixlng_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		FIXEDATOM_PUT(b,(b)->htype,p,h,H); \
		SIMPLE_PUT(b,lng,p,t,T); \
	} while (0)
#define fixlng_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		fixlng_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define fixlng_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		fixlng_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define fixdbl_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		FIXEDATOM_PUT(b,(b)->htype,p,h,H); \
		SIMPLE_PUT(b,dbl,p,t,T); \
	} while (0)
#define fixdbl_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		fixdbl_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define fixdbl_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		fixdbl_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define fixfix_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		FIXEDATOM_PUT(b,(b)->htype,p,h,H); \
		FIXEDATOM_PUT(b,(b)->ttype,p,t,T); \
	} while (0)
#define fixfix_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		fixfix_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define fixfix_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		fixfix_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define fixvar_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		FIXEDATOM_PUT(b,(b)->htype,p,h,H); \
		VARATOM_PUT(b,(b)->ttype,p,t,T); \
	} while (0)
#define fixvar_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		fixvar_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define fixvar_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		fixvar_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)








	
#define varvoid_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VARATOM_PUT(b,(b)->htype,p,h,H); \
		VOID_PUT(b,void,p,t,T); \
	} while (0)
#define varvoid_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		varvoid_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define varvoid_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		varvoid_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define varchr_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VARATOM_PUT(b,(b)->htype,p,h,H); \
		SIMPLE_PUT(b,chr,p,t,T); \
	} while (0)
#define varchr_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		varchr_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define varchr_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		varchr_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define varbte_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VARATOM_PUT(b,(b)->htype,p,h,H); \
		SIMPLE_PUT(b,bte,p,t,T); \
	} while (0)
#define varbte_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		varbte_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define varbte_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		varbte_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define varsht_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VARATOM_PUT(b,(b)->htype,p,h,H); \
		SIMPLE_PUT(b,sht,p,t,T); \
	} while (0)
#define varsht_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		varsht_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define varsht_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		varsht_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define varint_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VARATOM_PUT(b,(b)->htype,p,h,H); \
		SIMPLE_PUT(b,int,p,t,T); \
	} while (0)
#define varint_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		varint_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define varint_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		varint_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define varflt_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VARATOM_PUT(b,(b)->htype,p,h,H); \
		SIMPLE_PUT(b,flt,p,t,T); \
	} while (0)
#define varflt_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		varflt_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define varflt_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		varflt_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define varoid_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VARATOM_PUT(b,(b)->htype,p,h,H); \
		SIMPLE_PUT(b,oid,p,t,T); \
	} while (0)
#define varoid_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		varoid_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define varoid_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		varoid_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define varwrd_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VARATOM_PUT(b,(b)->htype,p,h,H); \
		SIMPLE_PUT(b,wrd,p,t,T); \
	} while (0)
#define varwrd_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		varwrd_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define varwrd_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		varwrd_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define varlng_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VARATOM_PUT(b,(b)->htype,p,h,H); \
		SIMPLE_PUT(b,lng,p,t,T); \
	} while (0)
#define varlng_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		varlng_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define varlng_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		varlng_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define vardbl_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VARATOM_PUT(b,(b)->htype,p,h,H); \
		SIMPLE_PUT(b,dbl,p,t,T); \
	} while (0)
#define vardbl_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		vardbl_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define vardbl_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		vardbl_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define varfix_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VARATOM_PUT(b,(b)->htype,p,h,H); \
		FIXEDATOM_PUT(b,(b)->ttype,p,t,T); \
	} while (0)
#define varfix_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		varfix_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define varfix_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		varfix_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)




	
#define varvar_bunfastins_nocheck_noinc(b, p, h, t)		\
	do {							\
		VARATOM_PUT(b,(b)->htype,p,h,H); \
		VARATOM_PUT(b,(b)->ttype,p,t,T); \
	} while (0)
#define varvar_bunfastins_nocheck(b, p, h, t, hs, ts)		\
	do {							\
		(b)->H->heap.free += hs;			\
		(b)->T->heap.free += ts;			\
		(b)->batCount ++;				\
		varvar_bunfastins_nocheck_noinc(b, p, h, t);	\
	} while (0)
#define varvar_bunfastins(b, h, t)					\
	do {								\
		register BUN _p = BUNlast(b);				\
		if (BATcount(b) + 1 > BATcapacity(b)){			\
			if (BATextend(b, BATgrows(b)) == NULL)  	\
				goto bunins_failed; 			\
		}							\
		varvar_bunfastins_nocheck(b, _p, h, t, Hsize(b), Tsize(b)); \
	} while (0)








/*
 * @-
 * For backward compatibility:
 * (also to be moved to gdk.mx and gdk_bat.mx, respectively)
 */

BAT *BAT_scanselect(BAT *b, BAT *bn, ptr tl, ptr th, bit li, bit hi, int equi, int nequi, int lval, int hval, int nocheck);

#endif /* _GDK_SCANSELECT_H */


